#!/bin/bash

# Check if an environment file was provided
if [ "$#" -ne 1 ]; then
  echo "Usage: $0 environment.yml"
  exit 1
fi

source filename_extractor.sh

ENV_FILE=$1
ENV_NAME=$(extract_filename $ENV_FILE)

# Create the Conda environment
echo "Creating Conda environment: $ENV_NAME"
conda env create -f $ENV_FILE -n $ENV_NAME

# Activate the environment
echo "Activating environment: $ENV_NAME"
source activate $ENV_NAME

# Pack the environment
echo "Packing environment: $ENV_NAME"
conda deactivate
conda pack -n $ENV_NAME -o ${ENV_NAME}.tar.gz

# Check if the file exists
if [ -f "${ENV_NAME}.tar.gz" ]; then
  echo "Environment $ENV_NAME packed successfully into ${ENV_NAME}.tar.gz"
else
    echo "Failed to pack the environment $ENV_NAME."
fi